--- orig/lisp/text-modes/hexl.el
+++ mod/lisp/text-modes/hexl.el
@@ -97,7 +97,7 @@
   :group 'hexl)
 
 (defcustom hexl-follow-ascii t
-  "If non-nil then highlight the ASCII character corresponding to point."
+  "If non-nil then highlight the ASCII/HEX character corresponding to point."
   :type 'boolean
   :group 'hexl
   :version "20.3")
@@ -106,7 +106,6 @@
   "Maximum offset into hexl buffer.")
 
 (defvar hexl-mode-map nil)
-
 (defvar hexl-mode-old-local-map)
 (defvar hexl-mode-old-mode-name)
 (defvar hexl-mode-old-major-mode)
@@ -117,6 +116,8 @@
 (defvar hexl-ascii-extent nil
   "Extent used to highlight ASCII element corresponding to current point.")
 (make-variable-buffer-local 'hexl-ascii-extent)
+(defvar hexl-current-address)
+(make-variable-buffer-local 'hexl-current-address)
 
 ;; routines
 
@@ -249,6 +250,10 @@
     (make-local-hook 'change-major-mode-hook)
     (add-hook 'change-major-mode-hook 'hexl-maybe-dehexlify-buffer nil t)
 
+    (make-local-variable 'modeline-format)
+    (setq modeline-format (append '("--" hexl-current-address "--")
+                                  (default-value 'modeline-format)))
+    
     (if hexl-follow-ascii (hexl-follow-ascii 1)))
   (run-hooks 'hexl-mode-hook))
 
@@ -352,6 +357,8 @@
 	     (if (>= current-column 41)
 		 (- current-column 41)
 	       (/ (- current-column  (/ current-column 5)) 2))))
+    (if (interactive-p)
+        (message "Current address 0x%010x = %10d" hexl-address hexl-address))
     hexl-address))
 
 (defun hexl-hex-position (address)
@@ -839,12 +846,22 @@
 	    )))))
 
 (defun hexl-follow-ascii-find ()
-  "Find and highlight the ASCII element corresponding to current point."
-  (let ((pos (+ 51
-		(- (point) (current-column))
-		(mod (hexl-current-address) 16))))
-    (set-extent-endpoints hexl-ascii-extent pos (1+ pos))
-    ))
+  "Find and highlight the ASCII/HEX element corresponding to current point."
+  (setq hexl-current-address (hexl-current-address))
+  (condition-case nil
+      (let ((col (current-column))
+	    (point (point))
+	    start end)
+	(if (> col 48)
+	    (setq start (+ 10 (- point col) (/ (- col 51) 2)
+			   (* 2 (mod (hexl-current-address) 16)))
+		  end (+ start 2))
+	  (setq start (+ 51 (- point col) (mod hexl-current-address 16))
+		end (+ start 1)))
+	(set-extent-endpoints hexl-ascii-extent start end))
+    (error nil))
+  (setq hexl-current-address
+        (format "0x%x=%d" hexl-current-address hexl-current-address)))
 
 ;; startup stuff.
 
